/*
 * Copyright (c) 2001-2004 MUSIC TECHNOLOGY GROUP (MTG)
 *                         UNIVERSITAT POMPEU FABRA
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <CLAM/ProcessingDefinitionAdapter.hxx>
#include <CLAM/Assert.hxx>
#include <CLAM/Processing.hxx>
#include <CLAM/ProcessingConfig.hxx>
#include <CLAM/ProcessingFactory.hxx>
#include <CLAM/XMLAdapter.hxx>
#include <CLAM/XmlStorageErr.hxx>

namespace CLAM
{
	ProcessingDefinitionAdapter::ProcessingDefinitionAdapter( Processing * adaptee, const std::string & name, const std::string & position, const std::string & size )
		:  mAdaptee(adaptee), mName(name), mPosition(position), mSize(size)
	{
	}

	ProcessingDefinitionAdapter::~ProcessingDefinitionAdapter()
	{
	}

	void ProcessingDefinitionAdapter::StoreOn (Storage & store) const
	{
		Text nameCopy(mName);
		XMLAdapter<Text> nameAdapter( nameCopy, "id");
		Text className(mAdaptee->GetClassName());
		XMLAdapter<Text> classNameAdapter( className, "type");
		store.Store(nameAdapter);
		store.Store(classNameAdapter);
		if (mPosition!="") // check if position is defined (QPoint values are integers)
		{
			Text positionCopy(mPosition);
			XMLAdapter<Text> positionAdapter (positionCopy, "position");
			store.Store(positionAdapter);
		}
		if (mSize!="") // check if size is defined
		{
			Text sizeCopy(mSize);
			XMLAdapter<Text> sizeAdapter (sizeCopy, "size");
			store.Store(sizeAdapter);
		}
		XMLComponentAdapter configAdapter((ProcessingConfig&)mAdaptee->GetConfig());
		store.Store(configAdapter);
	}

	void ProcessingDefinitionAdapter::LoadFrom (Storage & store) 
	{	
		XMLAdapter<Text> nameAdapter( mName, "id");
		store.Load(nameAdapter);
		Text className("");
		XMLAdapter<Text> classNameAdapter( className, "type");
		store.Load(classNameAdapter);

		XMLAdapter<Text> positionAdapter (mPosition, "position");
		store.Load(positionAdapter);
		XMLAdapter<Text> sizeAdapter (mSize, "size");
		store.Load(sizeAdapter);

		try
		{
			mAdaptee = ProcessingFactory::GetInstance().CreateSafe(className);
		} catch (ErrFactory & e)
		{
			std::string msg = 
				"Error creating a processing object of type '" +
				className + "' which is not available.";
			throw XmlStorageErr(msg);
		}
		ProcessingConfig&  cfg = (ProcessingConfig&)mAdaptee->GetConfig();
		XMLComponentAdapter configAdapter( cfg );
		store.Load(configAdapter);
		mAdaptee->Configure(cfg);
	}
} // namespace CLAM

